@@ -1,25 +1,5 @@
 package SQL::Translator::Parser::MySQL;
 
-# -------------------------------------------------------------------
-# $Id: MySQL.pm,v 1.58 2007-03-19 17:15:24 duality72 Exp $
-# -------------------------------------------------------------------
-# Copyright (C) 2002-4 SQLFairy Authors
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2.
-#
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-# 02111-1307  USA
-# -------------------------------------------------------------------
-
 =head1 NAME
 
 SQL::Translator::Parser::MySQL - parser for MySQL
@@ -41,11 +21,11 @@ Here's the word from the MySQL site
 
   CREATE [TEMPORARY] TABLE [IF NOT EXISTS] tbl_name [(create_definition,...)]
   [table_options] [select_statement]
-  
+
   or
-  
+
   CREATE [TEMPORARY] TABLE [IF NOT EXISTS] tbl_name LIKE old_table_name;
-  
+
   create_definition:
     col_name type [NOT NULL | NULL] [DEFAULT default_value] [AUTO_INCREMENT]
               [PRIMARY KEY] [reference_definition]
@@ -57,7 +37,7 @@ Here's the word from the MySQL site
     or    [CONSTRAINT symbol] FOREIGN KEY [index_name] (index_col_name,...)
               [reference_definition]
     or    CHECK (expr)
-  
+
   type:
           TINYINT[(length)] [UNSIGNED] [ZEROFILL]
     or    SMALLINT[(length)] [UNSIGNED] [ZEROFILL]
@@ -86,24 +66,27 @@ Here's the word from the MySQL site
     or    LONGTEXT
     or    ENUM(value1,value2,value3,...)
     or    SET(value1,value2,value3,...)
-  
+
   index_col_name:
           col_name [(length)]
-  
+
   reference_definition:
           REFERENCES tbl_name [(index_col_name,...)]
                      [MATCH FULL | MATCH PARTIAL]
                      [ON DELETE reference_option]
                      [ON UPDATE reference_option]
-  
+
   reference_option:
           RESTRICT | CASCADE | SET NULL | NO ACTION | SET DEFAULT
-  
+
   table_options:
           TYPE = {BDB | HEAP | ISAM | InnoDB | MERGE | MRG_MYISAM | MYISAM }
+  or      ENGINE = {BDB | HEAP | ISAM | InnoDB | MERGE | MRG_MYISAM | MYISAM }
   or      AUTO_INCREMENT = #
   or      AVG_ROW_LENGTH = #
+  or      [ DEFAULT ] CHARACTER SET charset_name
   or      CHECKSUM = {0 | 1}
+  or      COLLATE collation_name
   or      COMMENT = "string"
   or      MAX_ROWS = #
   or      MIN_ROWS = #
@@ -117,6 +100,7 @@ Here's the word from the MySQL site
   or      DATA DIRECTORY="absolute path to directory"
   or      INDEX DIRECTORY="absolute path to directory"
 
+
 A subset of the ALTER TABLE syntax that allows addition of foreign keys:
 
   ALTER [IGNORE] TABLE tbl_name alter_specification [, alter_specification] ...
@@ -130,47 +114,60 @@ A subset of INSERT that we ignore:
 
   INSERT anything
 
+=head1 ARGUMENTS
+
+This parser takes a single optional parser_arg C<mysql_parser_version>, which
+provides the desired version for the target database. Any statement in the processed
+dump file, that is commented with a version higher than the one supplied, will be stripped.
+
+Valid version specifiers for C<mysql_parser_version> are listed L<here|SQL::Translator::Utils/parse_mysql_version>
+
+More information about the MySQL comment-syntax: L<http://dev.mysql.com/doc/refman/5.0/en/comments.html>
+
+
 =cut
 
 use strict;
-use vars qw[ $DEBUG $VERSION $GRAMMAR @EXPORT_OK ];
-$VERSION = sprintf "%d.%02d", q$Revision: 1.58 $ =~ /(\d+)\.(\d+)/;
+use warnings;
+
+our $VERSION = '1.59';
+
+our $DEBUG;
 $DEBUG   = 0 unless defined $DEBUG;
 
 use Data::Dumper;
-use Parse::RecDescent;
-use Exporter;
 use Storable qw(dclone);
 use DBI qw(:sql_types);
+use SQL::Translator::Utils qw/parse_mysql_version ddl_parser_instance/;
+
 use base qw(Exporter);
+our @EXPORT_OK = qw(parse);
 
-our %type_mapping = (
-);
-
-@EXPORT_OK = qw(parse);
-
-# Enable warnings within the Parse::RecDescent module.
-$::RD_ERRORS = 1; # Make sure the parser dies when it encounters an error
-$::RD_WARN   = 1; # Enable warnings. This will warn on unused rules &c.
-$::RD_HINT   = 1; # Give out hints to help fix problems.
+our %type_mapping = ();
 
 use constant DEFAULT_PARSER_VERSION => 30000;
 
-$GRAMMAR = << 'END_OF_GRAMMAR';
+our $GRAMMAR = << 'END_OF_GRAMMAR';
 
-{ 
-    my ( $database_name, %tables, $table_order, @table_comments, %views, $view_order, %procedures, $proc_order );
+{
+    my ( $database_name, %tables, $table_order, @table_comments, %views,
+        $view_order, %procedures, $proc_order );
     my $delimiter = ';';
 }
 
 #
 # The "eofile" rule makes the parser fail if any "statement" rule
-# fails.  Otherwise, the first successful match by a "statement" 
+# fails.  Otherwise, the first successful match by a "statement"
 # won't cause the failure needed to know that the parse, as a whole,
 # failed. -ky
 #
-startrule : statement(s) eofile { 
-    { tables => \%tables, database_name => $database_name, views => \%views, procedures =>\%procedures } 
+startrule : statement(s) eofile {
+    {
+        database_name => $database_name,
+        tables        => \%tables,
+        views         => \%views,
+        procedures    => \%procedures,
+    }
 }
 
 eofile : /^\Z/
@@ -200,9 +197,13 @@ drop : /drop/i TABLE /[^;]+/ "$delimiter"
 drop : /drop/i WORD(s) "$delimiter"
     { @table_comments = () }
 
+bit:
+    /(b'[01]{1,64}')/ |
+    /(b"[01]{1,64}")/
+
 string :
-  # MySQL strings, unlike common SQL strings, can be double-quoted or 
-  # single-quoted, and you can escape the delmiters by doubling (but only the 
+  # MySQL strings, unlike common SQL strings, can be double-quoted or
+  # single-quoted, and you can escape the delmiters by doubling (but only the
   # delimiter) or by backslashing.
 
    /'(\\.|''|[^\\\'])*'/ |
@@ -211,9 +212,9 @@ string :
 
 nonstring : /[^;\'"]+/
 
-statement_body : (string | nonstring)(s?)
+statement_body : string | nonstring
 
-insert : /insert/i  statement_body "$delimiter"
+insert : /insert/i  statement_body(s?) "$delimiter"
 
 delimiter : /delimiter/i /[\S]+/
     { $delimiter = $item[2] }
@@ -225,7 +226,7 @@ alter : ALTER TABLE table_name alter_specification(s /,/) "$delimiter"
         my $table_name                       = $item{'table_name'};
     die "Cannot ALTER table '$table_name'; it does not exist"
         unless $tables{ $table_name };
-        for my $definition ( @{ $item[4] } ) { 
+        for my $definition ( @{ $item[4] } ) {
         $definition->{'extra'}->{'alter'} = 1;
         push @{ $tables{ $table_name }{'constraints'} }, $definition;
     }
@@ -238,8 +239,11 @@ create : CREATE /database/i WORD "$delimiter"
     { @table_comments = () }
 
 create : CREATE TEMPORARY(?) TABLE opt_if_not_exists(?) table_name '(' create_definition(s /,/) /(,\s*)?\)/ table_option(s?) "$delimiter"
-    { 
+    {
         my $table_name                       = $item{'table_name'};
+        die "There is more than one definition for $table_name"
+            if ($tables{$table_name});
+
         $tables{ $table_name }{'order'}      = ++$table_order;
         $tables{ $table_name }{'table_name'} = $table_name;
 
@@ -252,10 +256,10 @@ create : CREATE TEMPORARY(?) TABLE opt_if_not_exists(?) table_name '(' create_de
         for my $definition ( @{ $item[7] } ) {
             if ( $definition->{'supertype'} eq 'field' ) {
                 my $field_name = $definition->{'name'};
-                $tables{ $table_name }{'fields'}{ $field_name } = 
+                $tables{ $table_name }{'fields'}{ $field_name } =
                     { %$definition, order => $i };
                 $i++;
-        
+
                 if ( $definition->{'is_primary_key'} ) {
                     push @{ $tables{ $table_name }{'constraints'} },
                         {
@@ -296,7 +300,7 @@ create : CREATE UNIQUE(?) /(index|key)/i index_name /on/i table_name '(' field_n
         push @{ $tables{ $item{'table_name'} }{'indices'} },
             {
                 name   => $item[4],
-                type   => $item[2] ? 'unique' : 'normal',
+                type   => $item[2][0] ? 'unique' : 'normal',
                 fields => $item[8],
             }
         ;
@@ -313,7 +317,7 @@ create : CREATE PROCEDURE NAME not_delimiter "$delimiter"
         my $func_name = $item[3];
         my $owner = '';
         my $sql = "$item[1] $item[2] $item[3] $item[4]";
-        
+
         $procedures{ $func_name }{'order'}  = ++$proc_order;
         $procedures{ $func_name }{'name'}   = $func_name;
         $procedures{ $func_name }{'owner'}  = $owner;
@@ -323,20 +327,23 @@ create : CREATE PROCEDURE NAME not_delimiter "$delimiter"
 PROCEDURE : /procedure/i
     | /function/i
 
-create : CREATE algorithm /view/i NAME not_delimiter "$delimiter"
+create : CREATE replace(?) algorithm(?) /view/i NAME not_delimiter "$delimiter"
     {
         @table_comments = ();
-        my $view_name = $item[4];
-        my $sql = "$item[1] $item[2] $item[3] $item[4] $item[5]";
-        
+        my $view_name = $item[5];
+        my $sql = join(q{ }, grep { defined and length } $item[1], $item[2]->[0], $item[3]->[0])
+            . " $item[4] $item[5] $item[6]";
+
         # Hack to strip database from function calls in SQL
         $sql =~ s#`\w+`\.(`\w+`\()##g;
-        
+
         $views{ $view_name }{'order'}  = ++$view_order;
         $views{ $view_name }{'name'}   = $view_name;
         $views{ $view_name }{'sql'}    = $sql;
     }
 
+replace : /or replace/i
+
 algorithm : /algorithm/i /=/ WORD
     {
         $return = "$item[1]=$item[3]";
@@ -344,14 +351,14 @@ algorithm : /algorithm/i /=/ WORD
 
 not_delimiter : /.*?(?=$delimiter)/is
 
-create_definition : constraint 
+create_definition : constraint
     | index
     | field
     | comment
     | <error>
 
-comment : /^\s*(?:#|-{2}).*\n/ 
-    { 
+comment : /^\s*(?:#|-{2}).*\n/
+    {
         my $comment =  $item[1];
         $comment    =~ s/^\s*(#|--)\s*//;
         $comment    =~ s/\s*$//;
@@ -365,9 +372,9 @@ comment : /\/\*/ /.*?\*\//s
         $comment    =~ s/^\s*|\s*$//g;
         $return = $comment;
     }
-    
-field_comment : /^\s*(?:#|-{2}).*\n/ 
-    { 
+
+field_comment : /^\s*(?:#|-{2}).*\n/
+    {
         my $comment =  $item[1];
         $comment    =~ s/^\s*(#|--)\s*//;
         $comment    =~ s/\s*$//;
@@ -386,21 +393,21 @@ field_comment2 : /comment/i /'.*?'/
 blank : /\s*/
 
 field : field_comment(s?) field_name data_type field_qualifier(s?) field_comment2(?) reference_definition(?) on_update(?) field_comment(s?)
-    { 
+    {
         my %qualifiers  = map { %$_ } @{ $item{'field_qualifier(s?)'} || [] };
         if ( my @type_quals = @{ $item{'data_type'}{'qualifiers'} || [] } ) {
             $qualifiers{ $_ } = 1 for @type_quals;
         }
 
-        my $null = defined $qualifiers{'not_null'} 
+        my $null = defined $qualifiers{'not_null'}
                    ? $qualifiers{'not_null'} : 1;
         delete $qualifiers{'not_null'};
 
         my @comments = ( @{ $item[1] }, @{ $item[5] }, @{ $item[8] } );
 
-        $return = { 
+        $return = {
             supertype   => 'field',
-            name        => $item{'field_name'}, 
+            name        => $item{'field_name'},
             data_type   => $item{'data_type'}{'type'},
             size        => $item{'data_type'}{'size'},
             list        => $item{'data_type'}{'list'},
@@ -408,46 +415,46 @@ field : field_comment(s?) field_name data_type field_qualifier(s?) field_comment
             constraints => $item{'reference_definition(?)'},
             comments    => [ @comments ],
             %qualifiers,
-        } 
+        }
     }
     | <error>
 
 field_qualifier : not_null
-    { 
-        $return = { 
+    {
+        $return = {
              null => $item{'not_null'},
-        } 
+        }
     }
 
 field_qualifier : default_val
-    { 
-        $return = { 
+    {
+        $return = {
              default => $item{'default_val'},
-        } 
+        }
     }
 
 field_qualifier : auto_inc
-    { 
-        $return = { 
+    {
+        $return = {
              is_auto_inc => $item{'auto_inc'},
-        } 
+        }
     }
 
 field_qualifier : primary_key
-    { 
-        $return = { 
+    {
+        $return = {
              is_primary_key => $item{'primary_key'},
-        } 
+        }
     }
 
 field_qualifier : unsigned
-    { 
-        $return = { 
+    {
+        $return = {
              is_unsigned => $item{'unsigned'},
-        } 
+        }
     }
 
-field_qualifier : /character set/i WORD 
+field_qualifier : /character set/i WORD
     {
         $return = {
             'CHARACTER SET' => $item[2],
@@ -501,22 +508,23 @@ match_type : /match full/i { 'full' }
 on_delete : /on delete/i reference_option
     { $item[2] }
 
-on_update : 
+on_update :
     /on update/i 'CURRENT_TIMESTAMP'
     { $item[2] }
     |
     /on update/i reference_option
     { $item[2] }
 
-reference_option: /restrict/i | 
-    /cascade/i   | 
-    /set null/i  | 
-    /no action/i | 
+reference_option: /restrict/i |
+    /cascade/i   |
+    /set null/i  |
+    /no action/i |
     /set default/i
-    { $item[1] }  
+    { $item[1] }
 
 index : normal_index
     | fulltext_index
+    | spatial_index
     | <error>
 
 table_name   : NAME
@@ -526,7 +534,7 @@ field_name   : NAME
 index_name   : NAME
 
 data_type    : WORD parens_value_list(s?) type_qualifier(s?)
-    { 
+    {
         my $type = $item[1];
         my $size; # field size, applicable only to non-set fields
         my $list; # set list, applicable only to sets (duh)
@@ -541,12 +549,12 @@ data_type    : WORD parens_value_list(s?) type_qualifier(s?)
         }
 
 
-        $return        = { 
+        $return        = {
             type       => $type,
             size       => $size,
             list       => $list,
             qualifiers => $item[3],
-        } 
+        }
     }
 
 parens_field_list : '(' field_name(s /,/) ')'
@@ -562,7 +570,7 @@ field_type   : WORD
 
 create_index : /create/i /index/i
 
-not_null     : /not/i /null/i 
+not_null     : /not/i /null/i
     { $return = 0 }
     |
     /null/i
@@ -570,21 +578,26 @@ not_null     : /not/i /null/i
 
 unsigned     : /unsigned/i { $return = 0 }
 
-#default_val  : /default/i /(?:')?[\s\w\d:.-]*(?:')?/ 
-#    { 
-#        $item[2] =~ s/'//g; 
-#        $return  =  $item[2];
-#    }
-
-default_val : 
+default_val :
     /default/i 'CURRENT_TIMESTAMP'
     {
-        $return =  $item[2];
+        $return =  \$item[2];
     }
     |
-    /default/i /'(?:.*?\\')*.*?'|(?:')?[\w\d:.-]*(?:')?/
+    /default/i string
+    {
+        $item[2] =~ s/^\s*'|'\s*$//g or $item[2] =~ s/^\s*"|"\s*$//g;
+        $return  =  $item[2];
+    }
+    |
+    /default/i bit
+    {
+        $item[2] =~ s/b['"]([01]+)['"]/$1/g;
+        $return  =  $item[2];
+    }
+    |
+    /default/i /[\w\d:.-]+/
     {
-        $item[2] =~ s/^\s*'|'\s*$//g;
         $return  =  $item[2];
     }
 
@@ -623,50 +636,78 @@ foreign_key_def_begin : /constraint/i /foreign key/i WORD
     /foreign key/i
     { $return = '' }
 
-primary_key_def : primary_key index_name(?) '(' name_with_opt_paren(s /,/) ')'
-    { 
-        $return       = { 
+primary_key_def : primary_key index_type(?) '(' name_with_opt_paren(s /,/) ')' index_type(?)
+    {
+        $return       = {
             supertype => 'constraint',
-            name      => $item{'index_name(?)'}[0],
             type      => 'primary_key',
             fields    => $item[4],
+            options   => $item[2][0] || $item[6][0],
+        };
+    }
+    # In theory, and according to the doc, names should not be allowed here, but
+    # MySQL accept (and ignores) them, so we are not going to be less :)
+    | primary_key index_name_not_using(?) '(' name_with_opt_paren(s /,/) ')' index_type(?)
+    {
+        $return       = {
+            supertype => 'constraint',
+            type      => 'primary_key',
+            fields    => $item[4],
+            options   => $item[6][0],
         };
     }
 
-unique_key_def : UNIQUE KEY(?) index_name(?) '(' name_with_opt_paren(s /,/) ')'
-    { 
-        $return       = { 
+unique_key_def : UNIQUE KEY(?) index_name_not_using(?) index_type(?) '(' name_with_opt_paren(s /,/) ')' index_type(?)
+    {
+        $return       = {
             supertype => 'constraint',
-            name      => $item{'index_name(?)'}[0],
+            name      => $item[3][0],
             type      => 'unique',
-            fields    => $item[5],
-        } 
+            fields    => $item[6],
+            options   => $item[4][0] || $item[8][0],
+        }
     }
 
-normal_index : KEY index_name(?) '(' name_with_opt_paren(s /,/) ')'
-    { 
-        $return       = { 
+normal_index : KEY index_name_not_using(?) index_type(?) '(' name_with_opt_paren(s /,/) ')' index_type(?)
+    {
+        $return       = {
             supertype => 'index',
             type      => 'normal',
-            name      => $item{'index_name(?)'}[0],
-            fields    => $item[4],
-        } 
+            name      => $item[2][0],
+            fields    => $item[5],
+            options   => $item[3][0] || $item[7][0],
+        }
     }
 
+index_name_not_using : QUOTED_NAME
+    | /(\b(?!using)\w+\b)/ { $return = ($1 =~ /^using/i) ? undef : $1 }
+
+index_type : /using (btree|hash|rtree)/i { $return = uc $1 }
+
 fulltext_index : /fulltext/i KEY(?) index_name(?) '(' name_with_opt_paren(s /,/) ')'
-    { 
-        $return       = { 
+    {
+        $return       = {
             supertype => 'index',
             type      => 'fulltext',
             name      => $item{'index_name(?)'}[0],
             fields    => $item[5],
-        } 
+        }
+    }
+
+spatial_index : /spatial/i KEY(?) index_name(?) '(' name_with_opt_paren(s /,/) ')'
+    {
+        $return       = {
+            supertype => 'index',
+            type      => 'spatial',
+            name      => $item{'index_name(?)'}[0],
+            fields    => $item[5],
+        }
     }
 
 name_with_opt_paren : NAME parens_value_list(s?)
     { $item[2][0] ? "$item[1]($item[2][0][0])" : $item[1] }
 
-UNIQUE : /unique/i { 1 }
+UNIQUE : /unique/i
 
 KEY : /key/i | /index/i
 
@@ -677,15 +718,29 @@ table_option : /comment/i /=/ /'.*?'/
         $comment    =~ s/'$//;
         $return     = { comment => $comment };
     }
-    | /(default )?(charset|character set)/i /\s*=\s*/ WORD
-    { 
+    | /(default )?(charset|character set)/i /\s*=?\s*/ WORD
+    {
         $return = { 'CHARACTER SET' => $item[3] };
     }
-    | WORD /\s*=\s*/ WORD
-    { 
+    | /collate/i WORD
+    {
+        $return = { 'COLLATE' => $item[2] }
+    }
+    | /union/i /\s*=\s*/ '(' table_name(s /,/) ')'
+    {
+        $return = { $item[1] => $item[4] };
+    }
+    | WORD /\s*=\s*/ MAYBE_QUOTED_WORD
+    {
         $return = { $item[1] => $item[3] };
     }
-    
+
+MAYBE_QUOTED_WORD: /\w+/
+                 | /'(\w+)'/
+                 { $return = $1 }
+                 | /"(\w+)"/
+                 { $return = $1 }
+
 default : /default/i
 
 ADD : /add/i
@@ -704,16 +759,23 @@ DIGITS : /\d+/
 
 COMMA : ','
 
-NAME    : "`" /\w+/ "`"
-    { $item[2] }
-    | /\w+/
-    { $item[1] }
+BACKTICK : '`'
 
-VALUE   : /[-+]?\.?\d+(?:[eE]\d+)?/
+DOUBLE_QUOTE: '"'
+
+QUOTED_NAME : BACKTICK /[^`]+/ BACKTICK
+    { $item[2] }
+    | DOUBLE_QUOTE /[^"]+/ DOUBLE_QUOTE
+    { $item[2] }
+
+NAME: QUOTED_NAME
+    | /\w+/
+
+VALUE : /[-+]?\.?\d+(?:[eE]\d+)?/
     { $item[1] }
-    | /'.*?'/   
-    { 
-        # remove leading/trailing quotes 
+    | /'.*?'/
+    {
+        # remove leading/trailing quotes
         my $val = $item[1];
         $val    =~ s/^['"]|['"]$//g;
         $return = $val;
@@ -724,25 +786,33 @@ VALUE   : /[-+]?\.?\d+(?:[eE]\d+)?/
 CURRENT_TIMESTAMP : /current_timestamp(\(\))?/i
     | /now\(\)/i
     { 'CURRENT_TIMESTAMP' }
-    
+
 END_OF_GRAMMAR
 
-# -------------------------------------------------------------------
 sub parse {
     my ( $translator, $data ) = @_;
-    my $parser = Parse::RecDescent->new($GRAMMAR);
+
+    # Enable warnings within the Parse::RecDescent module.
+    local $::RD_ERRORS = 1 unless defined $::RD_ERRORS; # Make sure the parser dies when it encounters an error
+    local $::RD_WARN   = 1 unless defined $::RD_WARN; # Enable warnings. This will warn on unused rules &c.
+    local $::RD_HINT   = 1 unless defined $::RD_HINT; # Give out hints to help fix problems.
 
     local $::RD_TRACE  = $translator->trace ? 1 : undef;
     local $DEBUG       = $translator->debug;
 
-    unless (defined $parser) {
-        return $translator->error("Error instantiating Parse::RecDescent ".
-            "instance: Bad grammer");
+    my $parser = ddl_parser_instance('MySQL');
+
+    # Preprocess for MySQL-specific and not-before-version comments
+    # from mysqldump
+    my $parser_version = parse_mysql_version(
+        $translator->parser_args->{mysql_parser_version}, 'mysql'
+    ) || DEFAULT_PARSER_VERSION;
+
+    while ( $data =~
+        s#/\*!(\d{5})?(.*?)\*/#($1 && $1 > $parser_version ? '' : $2)#es
+    ) {
+        # do nothing; is there a better way to write this? -- ky
     }
-    
-    # Preprocess for MySQL-specific and not-before-version comments from mysqldump
-    my $parser_version = $translator->parser_args->{mysql_parser_version} || DEFAULT_PARSER_VERSION;
-    while ( $data =~ s#/\*!(\d{5})?(.*?)\*/#($1 && $1 > $parser_version ? '' : $2)#es ) {}
 
     my $result = $parser->startrule($data);
     return $translator->error( "Parse failed." ) unless defined $result;
@@ -751,22 +821,22 @@ sub parse {
     my $schema = $translator->schema;
     $schema->name($result->{'database_name'}) if $result->{'database_name'};
 
-    my @tables = sort { 
-        $result->{'tables'}{ $a }{'order'} 
-        <=> 
+    my @tables = sort {
+        $result->{'tables'}{ $a }{'order'}
+        <=>
         $result->{'tables'}{ $b }{'order'}
     } keys %{ $result->{'tables'} };
 
     for my $table_name ( @tables ) {
         my $tdata =  $result->{tables}{ $table_name };
-        my $table =  $schema->add_table( 
+        my $table =  $schema->add_table(
             name  => $tdata->{'table_name'},
         ) or die $schema->error;
 
         $table->comments( $tdata->{'comments'} );
 
-        my @fields = sort { 
-            $tdata->{'fields'}->{$a}->{'order'} 
+        my @fields = sort {
+            $tdata->{'fields'}->{$a}->{'order'}
             <=>
             $tdata->{'fields'}->{$b}->{'order'}
         } keys %{ $tdata->{'fields'} };
@@ -826,7 +896,23 @@ sub parse {
         }
 
         if ( my @options = @{ $tdata->{'table_options'} || [] } ) {
-            $table->options( \@options ) or die $table->error;
+            my @cleaned_options;
+            my @ignore_opts = $translator->parser_args->{'ignore_opts'}
+                ? split( /,/, $translator->parser_args->{'ignore_opts'} )
+                : ();
+            if (@ignore_opts) {
+                my $ignores = { map { $_ => 1 } @ignore_opts };
+                foreach my $option (@options) {
+                    # make sure the option isn't in ignore list
+                    my ($option_key) = keys %$option;
+                    if ( !exists $ignores->{$option_key} ) {
+                        push @cleaned_options, $option;
+                    }
+                }
+            } else {
+                @cleaned_options = @options;
+            }
+            $table->options( \@cleaned_options ) or die $table->error;
         }
 
         for my $cdata ( @{ $tdata->{'constraints'} || [] } ) {
@@ -837,65 +923,74 @@ sub parse {
                 reference_table  => $cdata->{'reference_table'},
                 reference_fields => $cdata->{'reference_fields'},
                 match_type       => $cdata->{'match_type'} || '',
-                on_delete        => $cdata->{'on_delete'} || $cdata->{'on_delete_do'},
-                on_update        => $cdata->{'on_update'} || $cdata->{'on_update_do'},
+                on_delete        => $cdata->{'on_delete'}
+                                 || $cdata->{'on_delete_do'},
+                on_update        => $cdata->{'on_update'}
+                                 || $cdata->{'on_update_do'},
             ) or die $table->error;
         }
 
-        # After the constrains and PK/idxs have been created, we normalize fields
+        # After the constrains and PK/idxs have been created,
+        # we normalize fields
         normalize_field($_) for $table->get_fields;
     }
-    
-    my @procedures = sort { 
-        $result->{procedures}->{ $a }->{'order'} <=> $result->{procedures}->{ $b }->{'order'}
+
+    my @procedures = sort {
+        $result->{procedures}->{ $a }->{'order'}
+        <=>
+        $result->{procedures}->{ $b }->{'order'}
     } keys %{ $result->{procedures} };
-    foreach my $proc_name (@procedures) {
+
+    for my $proc_name ( @procedures ) {
         $schema->add_procedure(
             name  => $proc_name,
             owner => $result->{procedures}->{$proc_name}->{owner},
             sql   => $result->{procedures}->{$proc_name}->{sql},
         );
     }
-
-    my @views = sort { 
-        $result->{views}->{ $a }->{'order'} <=> $result->{views}->{ $b }->{'order'}
+    my @views = sort {
+        $result->{views}->{ $a }->{'order'}
+        <=>
+        $result->{views}->{ $b }->{'order'}
     } keys %{ $result->{views} };
-    foreach my $view_name (keys %{ $result->{views} }) {
+
+    for my $view_name ( @views ) {
         $schema->add_view(
             name => $view_name,
-            sql  => $result->{views}->{$view_name}->{sql},
+            sql  => $result->{'views'}->{$view_name}->{sql},
         );
     }
 
     return 1;
 }
 
-# Takes a field, and returns 
+# Takes a field, and returns
 sub normalize_field {
     my ($field) = @_;
-    my ($size, $type, $list, $changed) = @_;
-  
+    my ($size, $type, $list, $unsigned, $changed);
+
     $size = $field->size;
     $type = $field->data_type;
     $list = $field->extra->{list} || [];
+    $unsigned = defined($field->extra->{unsigned});
 
     if ( !ref $size && $size eq 0 ) {
         if ( lc $type eq 'tinyint' ) {
-            $changed = $size != 4;
-            $size = 4;
+            $changed = $size != 4 - $unsigned;
+            $size = 4 - $unsigned;
         }
         elsif ( lc $type eq 'smallint' ) {
-            $changed = $size != 6;
-            $size = 6;
+            $changed = $size != 6 - $unsigned;
+            $size = 6 - $unsigned;
         }
         elsif ( lc $type eq 'mediumint' ) {
-            $changed = $size != 9;
-            $size = 9;
+            $changed = $size != 9 - $unsigned;
+            $size = 9 - $unsigned;
         }
         elsif ( $type =~ /^int(eger)?$/i ) {
-            $changed = $size != 11 || $type ne 'int';
+            $changed = $size != 11 - $unsigned || $type ne 'int';
             $type = 'int';
-            $size = 11;
+            $size = 11 - $unsigned;
         }
         elsif ( lc $type eq 'bigint' ) {
             $changed = $size != 20;
@@ -903,8 +998,10 @@ sub normalize_field {
         }
         elsif ( lc $type =~ /(float|double|decimal|numeric|real|fixed|dec)/ ) {
             my $old_size = (ref $size || '') eq 'ARRAY' ? $size : [];
-            $changed = @$old_size != 2 || $old_size->[0] != 8 || $old_size->[1] != 2;
-            $size = [8,2];
+            $changed     = @$old_size != 2
+                        || $old_size->[0] != 8
+                        || $old_size->[1] != 2;
+            $size        = [8,2];
         }
     }
 
@@ -924,6 +1021,7 @@ sub normalize_field {
         $changed = $size != 4_294_967_295;
         $size = 4_294_967_295;
     }
+
     if ( $field->data_type =~ /(set|enum)/i && !$field->size ) {
         my %extra = $field->extra;
         my $longest = 0;
@@ -935,20 +1033,21 @@ sub normalize_field {
     }
 
 
-    if ($changed) {
-      # We only want to clone the field, not *everything*
-      { local $field->{table} = undef;
-        $field->parsed_field(dclone($field));
-        $field->parsed_field->{table} = $field->table;
-      }
-      $field->size($size);
-      $field->data_type($type);
-      $field->sql_data_type( $type_mapping{lc $type} ) if exists $type_mapping{lc $type};
-      $field->extra->{list} = $list if @$list;
+    if ( $changed ) {
+        # We only want to clone the field, not *everything*
+        {
+            local $field->{table} = undef;
+            $field->parsed_field( dclone( $field ) );
+            $field->parsed_field->{table} = $field->table;
+        }
+        $field->size( $size );
+        $field->data_type( $type );
+        $field->sql_data_type( $type_mapping{ lc $type } )
+            if exists $type_mapping{ lc $type };
+        $field->extra->{list} = $list if @$list;
     }
 }
 
-
 1;
 
 # -------------------------------------------------------------------
