@@ -1,25 +1,5 @@
 package SQL::Translator::Schema::Table;
 
-# ----------------------------------------------------------------------
-# $Id: Table.pm,v 1.37 2007-10-24 10:55:44 schiffbruechige Exp $
-# ----------------------------------------------------------------------
-# Copyright (C) 2002-4 SQLFairy Authors
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2.
-#
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-# 02111-1307  USA
-# -------------------------------------------------------------------
-
 =pod
 
 =head1 NAME
@@ -31,7 +11,7 @@ SQL::Translator::Schema::Table - SQL::Translator table object
   use SQL::Translator::Schema::Table;
   my $table = SQL::Translator::Schema::Table->new( name => 'foo' );
 
-=head1 DESCSIPTION
+=head1 DESCRIPTION
 
 C<SQL::Translator::Schema::Table> is the table object.
 
@@ -40,6 +20,7 @@ C<SQL::Translator::Schema::Table> is the table object.
 =cut
 
 use strict;
+use warnings;
 use SQL::Translator::Utils 'parse_list_arg';
 use SQL::Translator::Schema::Constants;
 use SQL::Translator::Schema::Constraint;
@@ -49,7 +30,7 @@ use Data::Dumper;
 
 use base 'SQL::Translator::Schema::Object';
 
-use vars qw( $VERSION $FIELD_ORDER );
+our $VERSION = '1.59';
 
 # Stringify to our name, being careful not to pass any args through so we don't
 # accidentally set it to undef. We also have to tweak bool so the object is
@@ -60,8 +41,6 @@ use overload
     fallback => 1,
 ;
 
-# ----------------------------------------------------------------------
-
 __PACKAGE__->_attributes( qw/schema name comments options order/ );
 
 =pod
@@ -70,21 +49,32 @@ __PACKAGE__->_attributes( qw/schema name comments options order/ );
 
 Object constructor.
 
-  my $table  =  SQL::Translator::Schema::Table->new( 
+  my $table  =  SQL::Translator::Schema::Table->new(
       schema => $schema,
       name   => 'foo',
   );
 
 =cut
 
-# ----------------------------------------------------------------------
+sub new {
+  my $class = shift;
+  my $self = $class->SUPER::new (@_)
+    or return;
+
+  $self->{_order} = { map { $_ => 0 } qw/
+    field
+  /};
+
+  return $self;
+}
+
 sub add_constraint {
 
 =pod
 
 =head2 add_constraint
 
-Add a constraint to the table.  Returns the newly created 
+Add a constraint to the table.  Returns the newly created
 C<SQL::Translator::Schema::Constraint> object.
 
   my $c1     = $table->add_constraint(
@@ -109,7 +99,7 @@ C<SQL::Translator::Schema::Constraint> object.
     else {
         my %args = @_;
         $args{'table'} = $self;
-        $constraint = $constraint_class->new( \%args ) or 
+        $constraint = $constraint_class->new( \%args ) or
            return $self->error( $constraint_class->error );
     }
 
@@ -122,7 +112,7 @@ C<SQL::Translator::Schema::Constraint> object.
     if ( $pk && $constraint->type eq PRIMARY_KEY ) {
         $self->primary_key( $constraint->fields );
         $pk->name($constraint->name) if $constraint->name;
-        my %extra = $constraint->extra; 
+        my %extra = $constraint->extra;
         $pk->extra(%extra) if keys %extra;
         $constraint = $pk;
         $ok         = 0;
@@ -135,20 +125,20 @@ C<SQL::Translator::Schema::Constraint> object.
         }
     }
     #
-    # See if another constraint of the same type 
+    # See if another constraint of the same type
     # covers the same fields.  -- This doesn't work!  ky
     #
 #    elsif ( $constraint->type ne CHECK_C ) {
 #        my @field_names = $constraint->fields;
-#        for my $c ( 
-#            grep { $_->type eq $constraint->type } 
-#            $self->get_constraints 
+#        for my $c (
+#            grep { $_->type eq $constraint->type }
+#            $self->get_constraints
 #        ) {
 #            my %fields = map { $_, 1 } $c->fields;
 #            for my $field_name ( @field_names ) {
 #                if ( $fields{ $field_name } ) {
 #                    $constraint = $c;
-#                    $ok = 0; 
+#                    $ok = 0;
 #                    last;
 #                }
 #            }
@@ -163,7 +153,6 @@ C<SQL::Translator::Schema::Constraint> object.
     return $constraint;
 }
 
-# ----------------------------------------------------------------------
 sub drop_constraint {
 
 =pod
@@ -189,7 +178,7 @@ an index name or an C<SQL::Translator::Schema::Constraint> object.
         $constraint_name = shift;
     }
 
-    if ( ! grep { $_->name eq $constraint_name } @ { $self->{'constraints'} } ) { 
+    if ( ! grep { $_->name eq $constraint_name } @ { $self->{'constraints'} } ) {
         return $self->error(qq[Can't drop constraint: "$constraint_name" doesn't exist]);
     }
 
@@ -200,7 +189,6 @@ an index name or an C<SQL::Translator::Schema::Constraint> object.
     return $constraint;
 }
 
-# ----------------------------------------------------------------------
 sub add_index {
 
 =pod
@@ -232,7 +220,7 @@ C<SQL::Translator::Schema::Index> object.
     else {
         my %args = @_;
         $args{'table'} = $self;
-        $index = $index_class->new( \%args ) or return 
+        $index = $index_class->new( \%args ) or return
             $self->error( $index_class->error );
     }
     foreach my $ex_index ($self->get_indices) {
@@ -242,7 +230,6 @@ C<SQL::Translator::Schema::Index> object.
     return $index;
 }
 
-# ----------------------------------------------------------------------
 sub drop_index {
 
 =pod
@@ -268,7 +255,7 @@ an index name of an C<SQL::Translator::Schema::Index> object.
         $index_name = shift;
     }
 
-    if ( ! grep { $_->name eq  $index_name } @{ $self->{'indices'} }) { 
+    if ( ! grep { $_->name eq  $index_name } @{ $self->{'indices'} }) {
         return $self->error(qq[Can't drop index: "$index_name" doesn't exist]);
     }
 
@@ -279,7 +266,6 @@ an index name of an C<SQL::Translator::Schema::Index> object.
     return $index;
 }
 
-# ----------------------------------------------------------------------
 sub add_field {
 
 =pod
@@ -287,8 +273,8 @@ sub add_field {
 =head2 add_field
 
 Add an field to the table.  Returns the newly created
-C<SQL::Translator::Schema::Field> object.  The "name" parameter is 
-required.  If you try to create a field with the same name as an 
+C<SQL::Translator::Schema::Field> object.  The "name" parameter is
+required.  If you try to create a field with the same name as an
 existing field, you will get an error and the field will not be created.
 
   my $f1        =  $table->add_field(
@@ -297,8 +283,8 @@ existing field, you will get an error and the field will not be created.
       size      => 11,
   );
 
-  my $f2     =  SQL::Translator::Schema::Field->new( 
-      name   => 'name', 
+  my $f2     =  SQL::Translator::Schema::Field->new(
+      name   => 'name',
       table  => $table,
   );
   $f2 = $table->add_field( $field2 ) or die $table->error;
@@ -316,15 +302,15 @@ existing field, you will get an error and the field will not be created.
     else {
         my %args = @_;
         $args{'table'} = $self;
-        $field = $field_class->new( \%args ) or return 
+        $field = $field_class->new( \%args ) or return
             $self->error( $field_class->error );
     }
 
-    $field->order( ++$FIELD_ORDER );
+    $field->order( ++$self->{_order}{field} );
     # We know we have a name as the Field->new above errors if none given.
     my $field_name = $field->name;
 
-    if ( exists $self->{'fields'}{ $field_name } ) { 
+    if ( exists $self->{'fields'}{ $field_name } ) {
         return $self->error(qq[Can't create field: "$field_name" exists]);
     }
     else {
@@ -333,15 +319,15 @@ existing field, you will get an error and the field will not be created.
 
     return $field;
 }
-# ----------------------------------------------------------------------
+
 sub drop_field {
 
 =pod
 
 =head2 drop_field
 
-Remove a field from the table. Returns the field object if the field was 
-found and removed, an error otherwise. The single parameter can be either 
+Remove a field from the table. Returns the field object if the field was
+found and removed, an error otherwise. The single parameter can be either
 a field name or an C<SQL::Translator::Schema::Field> object.
 
   $table->drop_field('myfield');
@@ -386,14 +372,13 @@ a field name or an C<SQL::Translator::Schema::Field> object.
     return $field;
 }
 
-# ----------------------------------------------------------------------
 sub comments {
 
 =pod
 
 =head2 comments
 
-Get or set the comments on a table.  May be called several times to 
+Get or set the comments on a table.  May be called several times to
 set and it will accumulate the comments.  Called in an array context,
 returns each comment individually; called in a scalar context, returns
 all the comments joined on newlines.
@@ -413,17 +398,16 @@ all the comments joined on newlines.
     }
 
     if ( @{ $self->{'comments'} || [] } ) {
-        return wantarray 
+        return wantarray
             ? @{ $self->{'comments'} }
             : join( "\n", @{ $self->{'comments'} } )
         ;
-    } 
+    }
     else {
         return wantarray ? () : undef;
     }
 }
 
-# ----------------------------------------------------------------------
 sub get_constraints {
 
 =pod
@@ -439,7 +423,7 @@ Returns all the constraint objects as an array or array reference.
     my $self = shift;
 
     if ( ref $self->{'constraints'} ) {
-        return wantarray 
+        return wantarray
             ? @{ $self->{'constraints'} } : $self->{'constraints'};
     }
     else {
@@ -448,7 +432,6 @@ Returns all the constraint objects as an array or array reference.
     }
 }
 
-# ----------------------------------------------------------------------
 sub get_indices {
 
 =pod
@@ -464,8 +447,8 @@ Returns all the index objects as an array or array reference.
     my $self = shift;
 
     if ( ref $self->{'indices'} ) {
-        return wantarray 
-            ? @{ $self->{'indices'} } 
+        return wantarray
+            ? @{ $self->{'indices'} }
             : $self->{'indices'};
     }
     else {
@@ -474,7 +457,6 @@ Returns all the index objects as an array or array reference.
     }
 }
 
-# ----------------------------------------------------------------------
 sub get_field {
 
 =pod
@@ -491,18 +473,17 @@ Returns a field by the name provided.
     my $field_name = shift or return $self->error('No field name');
     my $case_insensitive = shift;
     if ( $case_insensitive ) {
-    	$field_name = uc($field_name);
-    	foreach my $field ( keys %{$self->{fields}} ) {
-    		return $self->{fields}{$field} if $field_name eq uc($field);
-    	}
-    	return $self->error(qq[Field "$field_name" does not exist]);
+      $field_name = uc($field_name);
+      foreach my $field ( keys %{$self->{fields}} ) {
+         return $self->{fields}{$field} if $field_name eq uc($field);
+      }
+      return $self->error(qq[Field "$field_name" does not exist]);
     }
     return $self->error( qq[Field "$field_name" does not exist] ) unless
         exists $self->{'fields'}{ $field_name };
     return $self->{'fields'}{ $field_name };
 }
 
-# ----------------------------------------------------------------------
 sub get_fields {
 
 =pod
@@ -516,7 +497,7 @@ Returns all the field objects as an array or array reference.
 =cut
 
     my $self = shift;
-    my @fields = 
+    my @fields =
         map  { $_->[1] }
         sort { $a->[0] <=> $b->[0] }
         map  { [ $_->order, $_ ] }
@@ -531,7 +512,6 @@ Returns all the field objects as an array or array reference.
     }
 }
 
-# ----------------------------------------------------------------------
 sub is_valid {
 
 =pod
@@ -548,8 +528,8 @@ Determine whether the view is valid or not.
     return $self->error('No name')   unless $self->name;
     return $self->error('No fields') unless $self->get_fields;
 
-    for my $object ( 
-        $self->get_fields, $self->get_indices, $self->get_constraints 
+    for my $object (
+        $self->get_fields, $self->get_indices, $self->get_constraints
     ) {
         return $object->error unless $object->is_valid;
     }
@@ -557,7 +537,6 @@ Determine whether the view is valid or not.
     return 1;
 }
 
-# ----------------------------------------------------------------------
 sub is_trivial_link {
 
 =pod
@@ -577,15 +556,15 @@ True if table has no data (non-key) fields and only uses single key joins.
     my %fk = ();
 
     foreach my $field ( $self->get_fields ) {
-	  next unless $field->is_foreign_key;
-	  $fk{$field->foreign_key_reference->reference_table}++;
-	}
+     next unless $field->is_foreign_key;
+     $fk{$field->foreign_key_reference->reference_table}++;
+   }
 
     foreach my $referenced (keys %fk){
-	if($fk{$referenced} > 1){
-	  $self->{'is_trivial_link'} = 0;
-	  last;
-	}
+   if($fk{$referenced} > 1){
+     $self->{'is_trivial_link'} = 0;
+     last;
+   }
     }
 
     return $self->{'is_trivial_link'};
@@ -617,7 +596,6 @@ Returns true if the table has some non-key fields.
     return $self->{'is_data'};
 }
 
-# ----------------------------------------------------------------------
 sub can_link {
 
 =pod
@@ -666,7 +644,7 @@ Determine whether the table can link two arg tables via many-to-many.
         $self->{'can_link'}{ $table1->name }{ $table2->name } =
           [ 'one2one', $fk{ $table2->name }, $fk{ $table1->name } ];
 
-        # non-trivial traversal.  one way to link table2, 
+        # non-trivial traversal.  one way to link table2,
         # many ways to link table1
     }
     elsif ( scalar( @{ $fk{ $table1->name } } > 1 )
@@ -677,7 +655,7 @@ Determine whether the table can link two arg tables via many-to-many.
         $self->{'can_link'}{ $table2->name }{ $table1->name } =
           [ 'one2many', $fk{ $table2->name }, $fk{ $table1->name } ];
 
-        # non-trivial traversal.  one way to link table1, 
+        # non-trivial traversal.  one way to link table1,
         # many ways to link table2
     }
     elsif ( scalar( @{ $fk{ $table1->name } } == 1 )
@@ -698,7 +676,7 @@ Determine whether the table can link two arg tables via many-to-many.
         $self->{'can_link'}{ $table2->name }{ $table1->name } =
           [ 'many2many', $fk{ $table2->name }, $fk{ $table1->name } ];
 
-        # one of the tables didn't export a key 
+        # one of the tables didn't export a key
         # to this table, no linking possible
     }
     else {
@@ -709,7 +687,6 @@ Determine whether the table can link two arg tables via many-to-many.
     return $self->{'can_link'}{ $table1->name }{ $table2->name };
 }
 
-# ----------------------------------------------------------------------
 sub name {
 
 =pod
@@ -742,7 +719,6 @@ that name and disallows the change if one exists (setting the error to
     return $self->{'name'} || '';
 }
 
-# ----------------------------------------------------------------------
 sub schema {
 
 =pod
@@ -765,7 +741,6 @@ Get or set the table's schema object.
     return $self->{'schema'};
 }
 
-# ----------------------------------------------------------------------
 sub primary_key {
 
 =pod
@@ -798,7 +773,7 @@ These are eqivalent:
     my $constraint;
     if ( @$fields ) {
         for my $f ( @$fields ) {
-            return $self->error(qq[Invalid field "$f"]) unless 
+            return $self->error(qq[Invalid field "$f"]) unless
                 $self->get_field($f);
         }
 
@@ -808,7 +783,7 @@ These are eqivalent:
                 $has_pk = 1;
                 $c->fields( @{ $c->fields }, @$fields );
                 $constraint = $c;
-            } 
+            }
         }
 
         unless ( $has_pk ) {
@@ -831,7 +806,6 @@ These are eqivalent:
     return;
 }
 
-# ----------------------------------------------------------------------
 sub options {
 
 =pod
@@ -858,7 +832,6 @@ an array or array reference.
     }
 }
 
-# ----------------------------------------------------------------------
 sub order {
 
 =pod
@@ -880,13 +853,12 @@ Get or set the table's order.
     return $self->{'order'} || 0;
 }
 
-# ----------------------------------------------------------------------
 sub field_names {
 
 =head2 field_names
 
 Read-only method to return a list or array ref of the field names. Returns undef
-or an empty list if the table has no fields set. Usefull if you want to
+or an empty list if the table has no fields set. Useful if you want to
 avoid the overload magic of the Field objects returned by the get_fields method.
 
   my @names = $constraint->field_names;
@@ -894,7 +866,7 @@ avoid the overload magic of the Field objects returned by the get_fields method.
 =cut
 
     my $self = shift;
-    my @fields = 
+    my @fields =
         map  { $_->name }
         sort { $a->order <=> $b->order }
         values %{ $self->{'fields'} || {} };
@@ -908,7 +880,6 @@ avoid the overload magic of the Field objects returned by the get_fields method.
     }
 }
 
-# ----------------------------------------------------------------------
 sub equals {
 
 =pod
@@ -924,7 +895,7 @@ Determines if this table is the same as another
     my $self = shift;
     my $other = shift;
     my $case_insensitive = shift;
-    
+
     return 0 unless $self->SUPER::equals($other);
     return 0 unless $case_insensitive ? uc($self->name) eq uc($other->name) : $self->name eq $other->name;
     return 0 unless $self->_compare_objects(scalar $self->options, scalar $other->options);
@@ -934,14 +905,14 @@ Determines if this table is the same as another
     # Go through our fields
     my %checkedFields;
     foreach my $field ( $self->get_fields ) {
-    	my $otherField = $other->get_field($field->name, $case_insensitive);
-    	return 0 unless $field->equals($otherField, $case_insensitive);
-    	$checkedFields{$field->name} = 1;
+      my $otherField = $other->get_field($field->name, $case_insensitive);
+      return 0 unless $field->equals($otherField, $case_insensitive);
+      $checkedFields{$field->name} = 1;
     }
     # Go through the other table's fields
     foreach my $otherField ( $other->get_fields ) {
-    	next if $checkedFields{$otherField->name};
-    	return 0;
+      next if $checkedFields{$otherField->name};
+      return 0;
     }
 
     # Constraints
@@ -949,24 +920,24 @@ Determines if this table is the same as another
     my %checkedConstraints;
 CONSTRAINT:
     foreach my $constraint ( $self->get_constraints ) {
-    	foreach my $otherConstraint ( $other->get_constraints ) {
-    		if ( $constraint->equals($otherConstraint, $case_insensitive) ) {
-    			$checkedConstraints{$otherConstraint} = 1;
-    			next CONSTRAINT;
-    		}
-    	}
-    	return 0;
+      foreach my $otherConstraint ( $other->get_constraints ) {
+         if ( $constraint->equals($otherConstraint, $case_insensitive) ) {
+            $checkedConstraints{$otherConstraint} = 1;
+            next CONSTRAINT;
+         }
+      }
+      return 0;
     }
     # Go through the other table's constraints
 CONSTRAINT2:
     foreach my $otherConstraint ( $other->get_constraints ) {
-    	next if $checkedFields{$otherConstraint};
-    	foreach my $constraint ( $self->get_constraints ) {
-    		if ( $otherConstraint->equals($constraint, $case_insensitive) ) {
-    			next CONSTRAINT2;
-    		}
-    	}
-    	return 0;
+      next if $checkedFields{$otherConstraint};
+      foreach my $constraint ( $self->get_constraints ) {
+         if ( $otherConstraint->equals($constraint, $case_insensitive) ) {
+            next CONSTRAINT2;
+         }
+      }
+      return 0;
     }
 
     # Indices
@@ -974,35 +945,33 @@ CONSTRAINT2:
     my %checkedIndices;
 INDEX:
     foreach my $index ( $self->get_indices ) {
-    	foreach my $otherIndex ( $other->get_indices ) {
-    		if ( $index->equals($otherIndex, $case_insensitive) ) {
-    			$checkedIndices{$otherIndex} = 1;
-    			next INDEX;
-    		}
-    	}
-    	return 0;
+      foreach my $otherIndex ( $other->get_indices ) {
+         if ( $index->equals($otherIndex, $case_insensitive) ) {
+            $checkedIndices{$otherIndex} = 1;
+            next INDEX;
+         }
+      }
+      return 0;
     }
     # Go through the other table's indices
 INDEX2:
     foreach my $otherIndex ( $other->get_indices ) {
-    	next if $checkedIndices{$otherIndex};
-    	foreach my $index ( $self->get_indices ) {
-    		if ( $otherIndex->equals($index, $case_insensitive) ) {
-    			next INDEX2;
-    		}
-    	}
-    	return 0;
+      next if $checkedIndices{$otherIndex};
+      foreach my $index ( $self->get_indices ) {
+         if ( $otherIndex->equals($index, $case_insensitive) ) {
+            next INDEX2;
+         }
+      }
+      return 0;
     }
 
-	return 1;
+   return 1;
 }
 
-# ----------------------------------------------------------------------
-
 =head1 LOOKUP METHODS
 
-The following are a set of shortcut methods for getting commonly used lists of 
-fields and constraints. They all return lists or array refs of Field or 
+The following are a set of shortcut methods for getting commonly used lists of
+fields and constraints. They all return lists or array refs of Field or
 Constraint objects.
 
 =over 4
@@ -1046,7 +1015,6 @@ sub pkey_fields {
     return wantarray ? @fields : \@fields;
 }
 
-# ----------------------------------------------------------------------
 sub fkey_fields {
     my $me = shift;
     my @fields;
@@ -1054,14 +1022,12 @@ sub fkey_fields {
     return wantarray ? @fields : \@fields;
 }
 
-# ----------------------------------------------------------------------
 sub nonpkey_fields {
     my $me = shift;
     my @fields = grep { !$_->is_primary_key } $me->get_fields;
     return wantarray ? @fields : \@fields;
 }
 
-# ----------------------------------------------------------------------
 sub data_fields {
     my $me = shift;
     my @fields =
@@ -1069,7 +1035,6 @@ sub data_fields {
     return wantarray ? @fields : \@fields;
 }
 
-# ----------------------------------------------------------------------
 sub unique_fields {
     my $me = shift;
     my @fields;
@@ -1077,21 +1042,18 @@ sub unique_fields {
     return wantarray ? @fields : \@fields;
 }
 
-# ----------------------------------------------------------------------
 sub unique_constraints {
     my $me = shift;
     my @cons = grep { $_->type eq UNIQUE } $me->get_constraints;
     return wantarray ? @cons : \@cons;
 }
 
-# ----------------------------------------------------------------------
 sub fkey_constraints {
     my $me = shift;
     my @cons = grep { $_->type eq FOREIGN_KEY } $me->get_constraints;
     return wantarray ? @cons : \@cons;
 }
 
-# ----------------------------------------------------------------------
 sub DESTROY {
     my $self = shift;
     undef $self->{'schema'}; # destroy cyclical reference
@@ -1102,13 +1064,11 @@ sub DESTROY {
 
 1;
 
-# ----------------------------------------------------------------------
-
 =pod
 
 =head1 AUTHORS
 
-Ken Y. Clark E<lt>kclark@cpan.orgE<gt>,
+Ken Youens-Clark E<lt>kclark@cpan.orgE<gt>,
 Allen Day E<lt>allenday@ucla.eduE<gt>.
 
 =cut
